/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "partialSphere.h"
#include "../newtEngine/scene.h"


PartialSphereMesh::PartialSphereMesh(float radius, float halfAngle, int nbPhi, int nbLambda, float texOffset, bool inside):
	Mesh("partialSphere", false, true, false, true), mRadius(radius), mHalfAngle(halfAngle), mTexOffset(texOffset), mInside(inside) {
					
	float meshLevel = scene->getGraphicOptions()->getMeshLevel();
	float value;
		
	value = (float) nbPhi * meshLevel;
	if(value < 4) {
		mNbPhi = 4;
	} else {
		mNbPhi = (int) value;
	}

	value = (float) nbLambda * meshLevel;
	if(value < 3) {
		mNbLambda = 3;
	} else {
		mNbLambda = (int) value;
	}	
}
	
PartialSphereMesh::~PartialSphereMesh() {
}
		
void PartialSphereMesh::initFirstParams() {
	geometry.nbVertices = (mNbPhi + 1) * ( mNbLambda + 1);
	
	geometry.nbIndexArrays = mNbLambda;
	geometry.primType = new GLenum[mNbLambda];
	for(int i=0;i<mNbLambda;i++) {
		geometry.primType[i] = GL_QUAD_STRIP;
	}
	
	geometry.nbIndices = new int[mNbLambda];
	for(int i=0;i<mNbLambda;i++) {
		geometry.nbIndices[i] = 2 * (mNbPhi+1);
	}	
}
		
void PartialSphereMesh::createVertices(Vector3 *ptr) {
	float lambda, phi;
	float lambdaMin = M_PI /2. - mHalfAngle;		
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = ((float) i/mNbLambda)*mHalfAngle + lambdaMin; 
		for(int j=0;j<mNbPhi;j++) {
			phi = ((float) j/mNbPhi)*2.*M_PI; 
			*ptr = mRadius * Vector3(cos(lambda)*sin(phi),sin(lambda),cos(lambda)*cos(phi));
	    	ptr ++;
		}
		*ptr = mRadius * Vector3(0.,sin(lambda),cos(lambda));
	    ptr ++;
	}
}

void PartialSphereMesh::createColors(Color *ptr) {

}

void PartialSphereMesh::createNormals(Vector3 *ptr) {
	float lambda, phi;
	float lambdaMin = M_PI /2. - mHalfAngle;		
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = ((float) i/mNbLambda)*mHalfAngle + lambdaMin; 
		for(int j=0;j<mNbPhi;j++) {
			phi = ((float) j/mNbPhi)*2.*M_PI; 
			if(mInside) {
				*ptr = Vector3(-cos(lambda)*sin(phi),-sin(lambda),-cos(lambda)*cos(phi));
			} else {
				*ptr = Vector3(cos(lambda)*sin(phi),sin(lambda),cos(lambda)*cos(phi));
			}
	    	ptr ++;
		}
		if(mInside) {	
			*ptr = Vector3(0.,-sin(lambda),-cos(lambda));
		} else {
			*ptr = Vector3(0.,sin(lambda),cos(lambda));
		}		
	    ptr ++;
	}
}

void PartialSphereMesh::createTexCoords(GLfloat *ptr) {
	float lambda, phi;
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = (float) i/mNbLambda ; 
		for(int j=0;j<mNbPhi;j++) {
			phi = (float) j/mNbPhi; 
			*ptr = phi;
	    	ptr ++;
			*ptr = mTexOffset + lambda * (1.-mTexOffset);
	    	ptr ++;
		}
		*ptr = 1.;
		ptr ++;
		*ptr = lambda;
		ptr ++;
	}
}

void PartialSphereMesh::createFogCoords(GLfloat *ptr) {

}
	
void PartialSphereMesh::createTangents(Vector3 *ptr) {
	float lambda, phi;
	float lambdaMin = M_PI /2. - mHalfAngle;		
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = ((float) i/mNbLambda)*mHalfAngle + lambdaMin; 
		for(int j=0;j<mNbPhi;j++) {
			phi = ((float) j/mNbPhi)*2.*M_PI; 
			if(mInside) {
				*ptr = Vector3(-cos(phi),0.,sin(phi));
			} else {
				*ptr = Vector3(cos(phi),0.,-sin(phi));
			}
	    	ptr ++;
		}
		if(mInside) {	
			*ptr = Vector3(-1.,0.,0.);
		} else {
			*ptr = Vector3(1.,0.,0.);
		}		
	    ptr ++;		
	}
}

		
void PartialSphereMesh::createIndices(int numArray, GLuint *ptr) {
	if(mInside) {	
		for(int j=0;j<=mNbPhi;j++) {
			*ptr = numArray * (mNbPhi+1) + j;
			ptr ++;
			*ptr = (numArray+1) * (mNbPhi+1) + j;
			ptr ++;
		}
		*ptr = numArray * (mNbPhi+1);
		ptr ++;
		*ptr = (numArray+1) * (mNbPhi+1);
		ptr ++;

	} else {
		for(int j=0;j<=mNbPhi;j++) {
			*ptr = (numArray+1) * (mNbPhi+1) + j;
			ptr ++;			
			*ptr = numArray * (mNbPhi+1) + j;
			ptr ++;
		}	
		*ptr = (numArray+1) * (mNbPhi+1);
		ptr ++;			
		*ptr = numArray * (mNbPhi+1);
		ptr ++;
	}
}
